//Opis:
//Manipulator do FMS-a wykorzystujący standardową transmisję RS232

//Wersja 1v0 - wersja podstawowa
//Wersja 1v1 - zmiana organizacji bufora - symbol "/n" został przeniesiony do ostatniego bajtu

//Stała przechowujaca informację o wielkości pustego bufora nadawczego dla RS232
const unsigned char tx_buff_empty = Serial.availableForWrite();

#define adc_lotki A0
#define adc_przepustnica A1
#define adc_s_wysokosci A2
#define adc_s_kierunku A3

#define lotki_en 6
#define s_kierunku_en 7

void setup() {
  // put your setup code here, to run once:

  // Wejscia analogowe - manetki
  pinMode(adc_lotki,INPUT);  // Lotki - wejscie A0 adc / patrz sekcja definicji
  pinMode(adc_przepustnica,INPUT);  // Przepustnica - wejscie A1 adc
  pinMode(adc_s_wysokosci,INPUT);  // Ster wysokosci - wejscie A2 adc
  pinMode(adc_s_kierunku,INPUT);  // Ster kierunku - wejscie A3 adc

  //Konfiguracja niewykorzystanych pinów analogowych A4 i A5
  pinMode(A4,INPUT_PULLUP);
  pinMode(A5,INPUT_PULLUP);

  //Konfiguracja żródła napięcia odniesienia ADC
  analogReference(EXTERNAL);

  //Przełącznik dezaktywacji lotki/nic/manetki
  pinMode(lotki_en,INPUT_PULLUP);
  pinMode(s_kierunku_en,INPUT_PULLUP);

  //Jawna konfiguracja niewykorzystywanych pinów cyfrowych

  pinMode(2,INPUT_PULLUP);
  pinMode(3,INPUT_PULLUP);
  pinMode(4,INPUT_PULLUP);
  pinMode(5,INPUT_PULLUP);
  
  pinMode(8,INPUT_PULLUP);
  pinMode(9,INPUT_PULLUP);
  pinMode(10,INPUT_PULLUP);
  pinMode(11,INPUT_PULLUP);
  pinMode(12,INPUT_PULLUP);
  pinMode(13,INPUT_PULLUP);
  
//Inicjalizacja transisji RS232
  Serial.begin(19200,SERIAL_8N1);
  
}

char bufor_tx[6]; // Bufor nadawania: ramka składa się z 6-ciu bajtów: 

                              // 0xFF - synchronizacja
                              // bajt 2 to '/n' - ulatwia obserwacje działania w monitorze portu szeregowego
                              // cztery kolejne bajty to cztery kanały: ster wysokości, kierunku, lotki i przepustnica
                              
unsigned int stan_poprzedni[4] = {512, 512, 512, 512}; //Bufor z wartosciami nastaw odczytanych przez ADC

boolean lotki = HIGH;       //Flagi: lotki = 1 - lotki aktywne
boolean s_kierunku = HIGH;  // s_kierunku = 1 - ster kierunku aktywny

void loop() {

  //Petle rozpoczynamy od odczytu stanu przelacznika lotki/ster kierunku

  lotki = digitalRead(lotki_en);
  s_kierunku = digitalRead(s_kierunku_en);  


  //Zasadnicza czesc programu wykonuje sie gdy bufor nadawczy jest pusty
  //Uwaga! Nadajemy 6x10 (dane + bit startu i stopu) symboli z prędkością 19200bit/sek -> dane powinny zostać przesłane w czasie około 3.1ms
  //W celu ograniczenia ilości danych przesylanych do PC, w przypadku gdy bufor nie jest pusty, nastepuje 25ms przerwa w działani - patrz "else{"...
  
  if (Serial.availableForWrite()==tx_buff_empty){  

    if (lotki == LOW) {
    
      //stan_poprzedni[0] = (stan_poprzedni[0] + int(analogRead(adc_lotki))) /2; //kanał lotek - gdy te sa nieaktywne stan_poprzedni[0] nie jest aktualizowany
      stan_poprzedni[1] = (stan_poprzedni[1] + int(analogRead(adc_przepustnica))) /2;
      stan_poprzedni[2] = (stan_poprzedni[2] + int(analogRead(adc_s_wysokosci))) /2;
      stan_poprzedni[3] = (stan_poprzedni[3] + int(analogRead(adc_s_kierunku))) /2;

    }

    else if (s_kierunku == LOW) {

      stan_poprzedni[0] = (stan_poprzedni[0] + int(analogRead(adc_lotki))) /2;
      stan_poprzedni[1] = (stan_poprzedni[1] + int(analogRead(adc_przepustnica))) /2;
      stan_poprzedni[2] = (stan_poprzedni[2] + int(analogRead(adc_s_wysokosci))) /2;
      //stan_poprzedni[3] = (stan_poprzedni[3] + int(analogRead(adc_s_kierunku))) /2; //kanał steru kieunku - gdy ten jest nieaktywny stan_poprzedni[3] nie jest aktualizowany
      
    }

    else {

        //Jesli lotki = 1 i s_kierunku = 1 wszystkie nastawy sa aktualizowane
      stan_poprzedni[0] = (stan_poprzedni[0] + int(analogRead(adc_lotki))) /2;
      stan_poprzedni[1] = (stan_poprzedni[1] + int(analogRead(adc_przepustnica))) /2;
      stan_poprzedni[2] = (stan_poprzedni[2] + int(analogRead(adc_s_wysokosci))) /2;
      stan_poprzedni[3] = (stan_poprzedni[3] + int(analogRead(adc_s_kierunku))) /2;
    }

  //Ladowanie do bufora
  bufor_tx[0] = 0xFF;
  bufor_tx[1] = map(stan_poprzedni[3],0,1024,254,0);  //Ster kierunku
  bufor_tx[2] = map(stan_poprzedni[2],0,1024,0,254);  //Ster wysokosci - nie wymaga inwersji co wynika z sposobu podłączenia potencjometrów pomiędzy Vref a GND na PCB 
  bufor_tx[3] = map(stan_poprzedni[0],0,1024,254,0);  //Lotki
  bufor_tx[4] = map(stan_poprzedni[1],0,1024,254,0);  //Przepstnica - UWAGA! zakres jest ograniczony do 254 / 0xFE gdyż symbol 0xFF zarezerwowany jest jako symbol synchronizujący
  bufor_tx[5] = '\n';

  Serial.write(bufor_tx,6);
  }
  
  else {
    //Jesli bufor jest "pelny" czekamy
    delay(25);
    }
  
}
